/////////////////////////////////////////////////////////////////////
// UVA TeamName RoboCup agent developed for the Simulator
// League of the RoboCup 2001 competition.
//
// This team based on the Mainz Rolling Brains RoboCup team from
// the 2000 competition.
//
// Copyright (C) 2001 Team Members
// of the UVA TeamName RoboCup Team.
//
// This program under the terms of the GNU General Public License as
// published by the Free Software Foundation; version 2 of the
// License. See the README file in the root directory of our archive.
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// sensor.h Wed Jun 20 2001
// Description: This file holds the structs that describe objects
//							in the SensorData classes
//////////////////////////////////////////////////////////////////////

#ifndef _SENSOR_H_
#define _SENSOR_H_

#include "constants.h"

// Distances/directions in a SensorData object may be invalid, i.e.
// we may only see the distance but not the direction etc.
// These constants are used to mark distances or directions as invalid

inline bool IsDistValid(double dist) {return dist > INVALID_DIST;}
inline bool IsDirValid(double dir) {return dir > INVALID_DIR;}
inline Angle leftDirLimit( Angle nodaDir )
  // returns minimum Angle that would still be quantized to nodaDir
{
  double ret;
  double nodaDir_deg = rint(RAD_TO_DEG( nodaDir ));
  if( nodaDir_deg <= 0 ) ret = nodaDir_deg-1;
  else ret = nodaDir_deg;
  return DEG_TO_RAD( ret );
}


inline Angle rightDirLimit( Angle nodaDir )
  // returns maximum Angle that would still be quantized to nodaDir
{
  double ret;
  double nodaDir_deg = rint(RAD_TO_DEG( nodaDir ));
  if( nodaDir_deg < 0 ) ret = nodaDir_deg;
  else ret = nodaDir_deg+1;
  return DEG_TO_RAD( ret );
}

inline double upperDistLimit( double nodaDist, double qstep )
  // returns maximum distance that would still be quantized to nodaDist
{
  // From the outer quantization, adding 0.05 to nodaDist gives the absolute
  // maximum for the possible values of the exp(...) expression.

  double d = nodaDist + 0.05;

  // Invert exp()

  if( d > 0 )
    d = log( d );
  else
    {
      cerr << "NODA'S ERROR: illigal quantization result from server !" << endl;
      return 1.0;   // senseless value (but d < 0 is senseless as well)
    }

  // As exp is monotonous, log(nodaDist+0.05) is the absolute upper limit
  // for the possible values of the inner quantization. But as these values
  // must also have precision qstep, we can restrict d further to the lower of
  // the two quantization limits of the inner quantization.

  d = floor( d / qstep ) * qstep;

  // Now invert the inner quantization to upper limit.

  d = d + 0.5 * qstep;

  // Finally invert log.

  return exp( d );
}


inline double lowerDistLimit( double nodaDist, double qstep )
  // returns minimum distance that would still be quantized to nodaDist
{
  // Like above, only choose lower limits when inverting quantizations,
  // and pick the upper limit when restricting the inner quantizations value.

  double d = nodaDist - 0.05;

  // distances are positive
  if( d <= EPS )
    return 0.0;

  // Invert exp()

  d = log(d);

  d = ceil( d / qstep ) * qstep;

  d = d - 0.5 * qstep;

  return exp( d );
}

struct SensorObject
{
  bool valid;		// true:  object seen
  // false: object not seen, entries are invalid
  double dist;
  double dir;		// In radians
  double dist_change;
  double dir_change;	// In radians
};

struct SensorPlayer : public SensorObject
{
  team_number   team;
  int           number;		// -1 = unknown
  goalie_player goalie;
  double        head_dir;      // in radians
  double        body_dir;      // in radians
};

struct SensorGoal : public SensorObject
{
  goal_type which;	// Which goal is it? (left/right)
  team_number team;	// Is it our goal or the enemy's?
};

struct SensorFlag : public SensorObject
{
  flag_type which;
};

struct SensorLine : public SensorObject
{
  line_type which;
};

// the structure that stores the commands together with their parameters
// this struct may be deprecated b/c only MRB's code used it
struct CycleAction {

  int time; // the timestep during which the command was sent to the server

  CycleCommand command; // the command sent

  double param1;    // params of the command, see above at command description
  double param2;
};


/// Coach Stuff, may move in the future
struct ExactObject {
  bool valid;		// true:  object seen
  // false: object not seen, entries are invalid
  double x;  // position
  double y;
  double x_vel; // velocities in direction of axes
  double y_vel;
};

struct ExactPlayer : public ExactObject {
  team_number   team;
  int           number;
  goalie_player goalie;
  double        head_dir;      // in radians
  double        body_dir;      // in radians
};

struct ExactGoal : public ExactObject {
  goal_type which;	// Which goal is it? (left/right)
  team_number team;	// Is it our goal or the enemy's?
};
/// end coach stuff

#endif
